@@ -1,12 +1,13 @@
 import torch.nn as nn
 from mmcv.cnn import ConvModule
+from mmcv.runner import BaseModule
 
 from ..builder import BACKBONES
 from ..utils import ResLayer
 from .resnet import BasicBlock
 
 
-class HourglassModule(nn.Module):
+class HourglassModule(BaseModule):
     """Hourglass Module for HourglassNet backbone.
 
     Generate module recursively and use BasicBlock as the base unit.
@@ -18,14 +19,17 @@ class HourglassModule(nn.Module):
         stage_blocks (list[int]): Number of sub-modules stacked in current and
             follow-up HourglassModule.
         norm_cfg (dict): Dictionary to construct and config norm layer.
+        init_cfg (dict or list[dict], optional): Initialization config dict.
+            Default: None
     """
 
     def __init__(self,
                  depth,
                  stage_channels,
                  stage_blocks,
-                 norm_cfg=dict(type='BN', requires_grad=True)):
-        super(HourglassModule, self).__init__()
+                 norm_cfg=dict(type='BN', requires_grad=True),
+                 init_cfg=None):
+        super(HourglassModule, self).__init__(init_cfg)
 
         self.depth = depth
 
@@ -78,7 +82,7 @@ class HourglassModule(nn.Module):
 
 
 @BACKBONES.register_module()
-class HourglassNet(nn.Module):
+class HourglassNet(BaseModule):
     """HourglassNet backbone.
 
     Stacked Hourglass Networks for Human Pose Estimation.
@@ -95,6 +99,9 @@ class HourglassNet(nn.Module):
             HourglassModule.
         feat_channel (int): Feature channel of conv after a HourglassModule.
         norm_cfg (dict): Dictionary to construct and config norm layer.
+        pretrained (str, optional): model pretrained path. Default: None
+        init_cfg (dict or list[dict], optional): Initialization config dict.
+            Default: None
 
     Example:
         >>> from mmdet.models import HourglassNet
@@ -115,8 +122,12 @@ class HourglassNet(nn.Module):
                  stage_channels=(256, 256, 384, 384, 384, 512),
                  stage_blocks=(2, 2, 2, 2, 2, 4),
                  feat_channel=256,
-                 norm_cfg=dict(type='BN', requires_grad=True)):
-        super(HourglassNet, self).__init__()
+                 norm_cfg=dict(type='BN', requires_grad=True),
+                 pretrained=None,
+                 init_cfg=None):
+        assert init_cfg is None, 'To prevent abnormal initialization ' \
+                                 'behavior, init_cfg is not allowed to be set'
+        super(HourglassNet, self).__init__(init_cfg)
 
         self.num_stacks = num_stacks
         assert self.num_stacks >= 1
@@ -161,17 +172,10 @@ class HourglassNet(nn.Module):
 
         self.relu = nn.ReLU(inplace=True)
 
-    def init_weights(self, pretrained=None):
-        """Init module weights.
-
-        We do nothing in this function because all modules we used
-        (ConvModule, BasicBlock and etc.) have default initialization, and
-        currently we don't provide pretrained model of HourglassNet.
-
-        Detector's __init__() will call backbone's init_weights() with
-        pretrained as input, so we keep this function.
-        """
+    def init_weights(self):
+        """Init module weights."""
         # Training Centripetal Model needs to reset parameters for Conv2d
+        super(HourglassNet, self).init_weights()
         for m in self.modules():
             if isinstance(m, nn.Conv2d):
                 m.reset_parameters()
