@@ -1,7 +1,10 @@
+import copy
+import warnings
+
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from mmcv.cnn import normal_init
+from mmcv import ConfigDict
 from mmcv.ops import nms
 
 from ..builder import HEADS
@@ -13,8 +16,20 @@ from .rpn_test_mixin import RPNTestMixin
 class GARPNHead(RPNTestMixin, GuidedAnchorHead):
     """Guided-Anchor-based RPN head."""
 
-    def __init__(self, in_channels, **kwargs):
-        super(GARPNHead, self).__init__(1, in_channels, **kwargs)
+    def __init__(self,
+                 in_channels,
+                 init_cfg=dict(
+                     type='Normal',
+                     layer='Conv2d',
+                     std=0.01,
+                     override=dict(
+                         type='Normal',
+                         name='conv_loc',
+                         std=0.01,
+                         bias_prob=0.01)),
+                 **kwargs):
+        super(GARPNHead, self).__init__(
+            1, in_channels, init_cfg=init_cfg, **kwargs)
 
     def _init_layers(self):
         """Initialize layers of the head."""
@@ -22,11 +37,6 @@ class GARPNHead(RPNTestMixin, GuidedAnchorHead):
             self.in_channels, self.feat_channels, 3, padding=1)
         super(GARPNHead, self)._init_layers()
 
-    def init_weights(self):
-        """Initialize weights of the head."""
-        normal_init(self.rpn_conv, std=0.01)
-        super(GARPNHead, self).init_weights()
-
     def forward_single(self, x):
         """Forward feature of a single scale level."""
 
@@ -69,6 +79,39 @@ class GARPNHead(RPNTestMixin, GuidedAnchorHead):
                            cfg,
                            rescale=False):
         cfg = self.test_cfg if cfg is None else cfg
+
+        cfg = copy.deepcopy(cfg)
+
+        # deprecate arguments warning
+        if 'nms' not in cfg or 'max_num' in cfg or 'nms_thr' in cfg:
+            warnings.warn(
+                'In rpn_proposal or test_cfg, '
+                'nms_thr has been moved to a dict named nms as '
+                'iou_threshold, max_num has been renamed as max_per_img, '
+                'name of original arguments and the way to specify '
+                'iou_threshold of NMS will be deprecated.')
+        if 'nms' not in cfg:
+            cfg.nms = ConfigDict(dict(type='nms', iou_threshold=cfg.nms_thr))
+        if 'max_num' in cfg:
+            if 'max_per_img' in cfg:
+                assert cfg.max_num == cfg.max_per_img, f'You ' \
+                    f'set max_num and max_per_img at the same time, ' \
+                    f'but get {cfg.max_num} ' \
+                    f'and {cfg.max_per_img} respectively' \
+                    'Please delete max_num which will be deprecated.'
+            else:
+                cfg.max_per_img = cfg.max_num
+        if 'nms_thr' in cfg:
+            assert cfg.nms.iou_threshold == cfg.nms_thr, f'You set ' \
+                f'iou_threshold in nms and ' \
+                f'nms_thr at the same time, but get ' \
+                f'{cfg.nms.iou_threshold} and {cfg.nms_thr}' \
+                f' respectively. Please delete the ' \
+                f'nms_thr which will be deprecated.'
+
+        assert cfg.nms.get('type', 'nms') == 'nms', 'GARPNHead only support ' \
+            'naive nms.'
+
         mlvl_proposals = []
         for idx in range(len(cls_scores)):
             rpn_cls_score = cls_scores[idx]
@@ -117,17 +160,18 @@ class GARPNHead(RPNTestMixin, GuidedAnchorHead):
                 proposals = proposals[valid_inds, :]
                 scores = scores[valid_inds]
             # NMS in current level
-            proposals, _ = nms(proposals, scores, cfg.nms_thr)
+            proposals, _ = nms(proposals, scores, cfg.nms.iou_threshold)
             proposals = proposals[:cfg.nms_post, :]
             mlvl_proposals.append(proposals)
         proposals = torch.cat(mlvl_proposals, 0)
-        if cfg.nms_across_levels:
+        if cfg.get('nms_across_levels', False):
             # NMS across multi levels
-            proposals, _ = nms(proposals[:, :4], proposals[:, -1], cfg.nms_thr)
-            proposals = proposals[:cfg.max_num, :]
+            proposals, _ = nms(proposals[:, :4], proposals[:, -1],
+                               cfg.nms.iou_threshold)
+            proposals = proposals[:cfg.max_per_img, :]
         else:
             scores = proposals[:, 4]
-            num = min(cfg.max_num, proposals.shape[0])
+            num = min(cfg.max_per_img, proposals.shape[0])
             _, topk_inds = scores.topk(num)
             proposals = proposals[topk_inds, :]
         return proposals
