@@ -1,8 +1,7 @@
 import torch
 import torch.nn as nn
-from mmcv.cnn import bias_init_with_prob, normal_init
 from mmcv.ops import DeformConv2d, MaskedConv2d
-from mmcv.runner import force_fp32
+from mmcv.runner import BaseModule, force_fp32
 
 from mmdet.core import (anchor_inside_flags, build_anchor_generator,
                         build_assigner, build_bbox_coder, build_sampler,
@@ -12,7 +11,7 @@ from ..builder import HEADS, build_loss
 from .anchor_head import AnchorHead
 
 
-class FeatureAdaption(nn.Module):
+class FeatureAdaption(BaseModule):
     """Feature Adaption Module.
 
     Feature Adaption Module is implemented based on DCN v1.
@@ -24,14 +23,21 @@ class FeatureAdaption(nn.Module):
         out_channels (int): Number of channels in the output feature map.
         kernel_size (int): Deformable conv kernel size.
         deform_groups (int): Deformable conv group size.
+        init_cfg (dict or list[dict], optional): Initialization config dict.
     """
 
     def __init__(self,
                  in_channels,
                  out_channels,
                  kernel_size=3,
-                 deform_groups=4):
-        super(FeatureAdaption, self).__init__()
+                 deform_groups=4,
+                 init_cfg=dict(
+                     type='Normal',
+                     layer='Conv2d',
+                     std=0.1,
+                     override=dict(
+                         type='Normal', name='conv_adaption', std=0.01))):
+        super(FeatureAdaption, self).__init__(init_cfg)
         offset_channels = kernel_size * kernel_size * 2
         self.conv_offset = nn.Conv2d(
             2, deform_groups * offset_channels, 1, bias=False)
@@ -43,10 +49,6 @@ class FeatureAdaption(nn.Module):
             deform_groups=deform_groups)
         self.relu = nn.ReLU(inplace=True)
 
-    def init_weights(self):
-        normal_init(self.conv_offset, std=0.1)
-        normal_init(self.conv_adaption, std=0.01)
-
     def forward(self, x, shape):
         offset = self.conv_offset(shape.detach())
         x = self.relu(self.conv_adaption(x, offset))
@@ -75,6 +77,11 @@ class GuidedAnchorHead(AnchorHead):
         square_anchor_generator (dict): Config dict for square generator
         anchor_coder (dict): Config dict for anchor coder
         bbox_coder (dict): Config dict for bbox coder
+        reg_decoded_bbox (bool): If true, the regression loss would be
+            applied directly on decoded bounding boxes, converting both
+            the predicted boxes and regression targets to absolute
+            coordinates format. Default False. It should be `True` when
+            using `IoULoss`, `GIoULoss`, or `DIoULoss` in the bbox head.
         deform_groups: (int): Group number of DCN in
             FeatureAdaption module.
         loc_filter_thr (float): Threshold to filter out unconcerned regions.
@@ -82,51 +89,57 @@ class GuidedAnchorHead(AnchorHead):
         loss_shape (dict): Config of anchor shape loss.
         loss_cls (dict): Config of classification loss.
         loss_bbox (dict): Config of bbox regression loss.
+        init_cfg (dict or list[dict], optional): Initialization config dict.
     """
 
     def __init__(
-        self,
-        num_classes,
-        in_channels,
-        feat_channels=256,
-        approx_anchor_generator=dict(
-            type='AnchorGenerator',
-            octave_base_scale=8,
-            scales_per_octave=3,
-            ratios=[0.5, 1.0, 2.0],
-            strides=[4, 8, 16, 32, 64]),
-        square_anchor_generator=dict(
-            type='AnchorGenerator',
-            ratios=[1.0],
-            scales=[8],
-            strides=[4, 8, 16, 32, 64]),
-        anchor_coder=dict(
-            type='DeltaXYWHBBoxCoder',
-            target_means=[.0, .0, .0, .0],
-            target_stds=[1.0, 1.0, 1.0, 1.0]
-        ),
-        bbox_coder=dict(
-            type='DeltaXYWHBBoxCoder',
-            target_means=[.0, .0, .0, .0],
-            target_stds=[1.0, 1.0, 1.0, 1.0]
-        ),
-        reg_decoded_bbox=False,
-        deform_groups=4,
-        loc_filter_thr=0.01,
-        train_cfg=None,
-        test_cfg=None,
-        loss_loc=dict(
-            type='FocalLoss',
-            use_sigmoid=True,
-            gamma=2.0,
-            alpha=0.25,
-            loss_weight=1.0),
-        loss_shape=dict(type='BoundedIoULoss', beta=0.2, loss_weight=1.0),
-        loss_cls=dict(
-            type='CrossEntropyLoss', use_sigmoid=True, loss_weight=1.0),
-        loss_bbox=dict(type='SmoothL1Loss', beta=1.0,
-                       loss_weight=1.0)):  # yapf: disable
-        super(AnchorHead, self).__init__()
+            self,
+            num_classes,
+            in_channels,
+            feat_channels=256,
+            approx_anchor_generator=dict(
+                type='AnchorGenerator',
+                octave_base_scale=8,
+                scales_per_octave=3,
+                ratios=[0.5, 1.0, 2.0],
+                strides=[4, 8, 16, 32, 64]),
+            square_anchor_generator=dict(
+                type='AnchorGenerator',
+                ratios=[1.0],
+                scales=[8],
+                strides=[4, 8, 16, 32, 64]),
+            anchor_coder=dict(
+                type='DeltaXYWHBBoxCoder',
+                target_means=[.0, .0, .0, .0],
+                target_stds=[1.0, 1.0, 1.0, 1.0]
+            ),
+            bbox_coder=dict(
+                type='DeltaXYWHBBoxCoder',
+                target_means=[.0, .0, .0, .0],
+                target_stds=[1.0, 1.0, 1.0, 1.0]
+            ),
+            reg_decoded_bbox=False,
+            deform_groups=4,
+            loc_filter_thr=0.01,
+            train_cfg=None,
+            test_cfg=None,
+            loss_loc=dict(
+                type='FocalLoss',
+                use_sigmoid=True,
+                gamma=2.0,
+                alpha=0.25,
+                loss_weight=1.0),
+            loss_shape=dict(type='BoundedIoULoss', beta=0.2, loss_weight=1.0),
+            loss_cls=dict(
+                type='CrossEntropyLoss', use_sigmoid=True, loss_weight=1.0),
+            loss_bbox=dict(type='SmoothL1Loss', beta=1.0,
+                           loss_weight=1.0),
+            init_cfg=dict(type='Normal', layer='Conv2d', std=0.01,
+                          override=dict(type='Normal',
+                                        name='conv_loc',
+                                        std=0.01,
+                                        bias_prob=0.01))):  # yapf: disable
+        super(AnchorHead, self).__init__(init_cfg)
         self.in_channels = in_channels
         self.num_classes = num_classes
         self.feat_channels = feat_channels
@@ -207,16 +220,6 @@ class GuidedAnchorHead(AnchorHead):
         self.conv_reg = MaskedConv2d(self.feat_channels, self.num_anchors * 4,
                                      1)
 
-    def init_weights(self):
-        normal_init(self.conv_cls, std=0.01)
-        normal_init(self.conv_reg, std=0.01)
-
-        bias_cls = bias_init_with_prob(0.01)
-        normal_init(self.conv_loc, std=0.01, bias=bias_cls)
-        normal_init(self.conv_shape, std=0.01)
-
-        self.feature_adaption.init_weights()
-
     def forward_single(self, x):
         loc_pred = self.conv_loc(x)
         shape_pred = self.conv_shape(x)
