@@ -1,4 +1,7 @@
 from .bfp import BFP
+from .channel_mapper import ChannelMapper
+from .dilated_encoder import DilatedEncoder
+from .fpg import FPG
 from .fpn import FPN
 from .fpn_carafe import FPN_CARAFE
 from .hrfpn import HRFPN
@@ -9,6 +12,6 @@ from .rfp import RFP
 from .yolo_neck import YOLOV3Neck
 
 __all__ = [
-    'FPN', 'BFP', 'HRFPN', 'NASFPN', 'FPN_CARAFE', 'PAFPN', 'NASFCOS_FPN',
-    'RFP', 'YOLOV3Neck'
+    'FPN', 'BFP', 'ChannelMapper', 'HRFPN', 'NASFPN', 'FPN_CARAFE', 'PAFPN',
+    'NASFCOS_FPN', 'RFP', 'YOLOV3Neck', 'FPG', 'DilatedEncoder'
 ]
